/**************************************************************************/
/*                 Svga with Allegro source file                          */
/*                                                                        */
/* Heavily derived from code by Marat Fayzullin.                          */
/* Modified BERO for NES emulator                                         */
/*                                                                        */
/*         Adapted by Zeograd (Olivier Jolly) for using Allegro           */
/*                                                                        */
/**************************************************************************/

#ifdef ALLEGRO
#undef VGA

#include "osd_init_machine.h"
#include "sys_dep.h"
#include "svgaallg.h"
#include "lang.h"

/* Standard and Allegro includes */

int init_normal_mode_normal();
int init_normal_mode_eagle();
int init_normal_mode_scanline();

int (*init_normal_mode[3])()=
     {
      osd_gfx_init_normal_mode,
      init_normal_mode_eagle,
      init_normal_mode_scanline
      };

void PutImage_normal();
void PutImage_eagle();
void PutImage_scanline();

void (*PutImage[3])()=
     {
      PutImage_normal,
      PutImage_eagle,
      PutImage_scanline
      };


BITMAP *XBuf;
// The screen buffer where we draw before blitting it on screen

// static int changepal;
// may be used later to make cleaner code (maybe faster)

// static int CPURunning;
// Is pce running (that was hard...)

BITMAP *EAGLE_buf=NULL;
// the buffer where we will put eagleized video

int gamepad_driver = 0;
// what kind of jypad must we have to handle

// UChar gamepad = 0;
// gamepad detected ?

char dump_snd = 0;
// Do we write sound to file

char synchro;
//  fond,  fond,  fond? (french joke ;)

extern unsigned message_delay;
// must we draw the message

int vwidth,vheight;
// size of visible part of the screen (I got troubles with allegro screen->? values!)

// int blit_x,blit_y;
// where must we blit the screen buffer on screen

int screen_blit_x,screen_blit_y;
// where on the screen we must blit XBuf

// BITMAP *MESSAGE_SPR=0;
// the bitmap containing the message to display (avoid textout each frame)

int vmode=0;
// video mode to use
// 1 -> first 224*240 then 256*240 then 256*256 then 320*240 and 320*200
// 2 -> first 256*240 then 256*256 then 320*240 and 320*200
// 3 -> first 256*256 then 320*240 and 320*200
// 4 -> first 320*240 and 320*200
// 5 -> only try 320*200
// 6 -> only 800*600
// If you think others modes would be interesting, let me know

extern SChar silent;
// use sound?

// void OnBreak(int Arg){ set_gfx_mode(GFX_TEXT,0,0,0,0); CPURunning=0; }
// formely used to answer to ctrl-C

void SetPalette(void)
{
   int i;
   RGB r;

   r.g=r.b=r.r=0x3f;
   osd_gfx_set_color(255,0x3f,0x3f,0x3f);

   for(i=0;i<255;i++)
      {
           r.r=(i&0x1c)<<1;
           r.g=(i&0xe0)>>2;
           r.b=(i&0x03)<<4;
           osd_gfx_set_color(i,
                             r.r,
                             r.g,
                             r.b);
       }

}

extern FILE * out_snd;

/* Adapted from code of Vlendr'it */

void PutImage_scanline(void)
{
 int dum;
 int i,y = 0;

 BITMAP *temp = create_bitmap(io.screen_w * 2, io.screen_h * 2);

 if (message_delay)
   draw_sprite(XBuf,OSD_MESSAGE_SPR,screen_blit_x+8,min(io.screen_h,vheight)-16);

   clear(temp);

   for(i = 0; i < io.screen_h; i++)
   {
      stretch_blit(XBuf, temp, screen_blit_x - 8, i, io.screen_w, 1, 0, y, io.screen_w << 1, 1);
      y += 2;
   } 

   /* blit the bitmap onto the screen */

   blit(temp, screen, 0, 0, blit_x ,
             blit_y, temp->w, temp->h);
   destroy_bitmap(temp);

 }


void PutImage_eagle(void)
{
 int dum;

 if (message_delay)
   draw_sprite(XBuf,OSD_MESSAGE_SPR,screen_blit_x+8,min(io.screen_h,vheight)-16);

     for (dum=0;dum<io.screen_h-1;dum++)
    {
      // Eagle-blit piccy to the screen 
      eagle (
             (unsigned long*)(XBuf->line[dum]+screen_blit_x - 8)                  // First line of piccy  (320 pixels)
            ,(unsigned long*)(XBuf->line[dum+1]+screen_blit_x - 8)              // 2nd line of piccy (320 pixels)
            ,io.screen_w
            ,EAGLE_buf->seg
            ,(int)EAGLE_buf->line[dum*2] //  + ( ( (x   <<1) * piccy->w ) << 1)           // First eagled line = 320x2 = 640 pixels
            ,(int)EAGLE_buf->line[1+(dum*2)] // + (( ( (x+1)<<1) * piccy->w ) << 1)       // 2nd eagled line = 320x2 = 640 pixels
           );    
    }

    blit(EAGLE_buf,screen,0,0,blit_x,blit_y,io.screen_w<<1,(io.screen_h-1)<<1/*480-2*/);

 }

void PutImage_normal(void)
{

 int dum;

 if (message_delay)
   draw_sprite(XBuf,OSD_MESSAGE_SPR,screen_blit_x + 8,min(io.screen_h,vheight)-16);
/*
 {
   PALETTE Pal;
   save_bitmap("output.bmp",XBuf, Pal);
  }
*/

 blit(XBuf,screen,screen_blit_x,screen_blit_y,blit_x,blit_y - 8,io.screen_w,io.screen_h);

}

/* TrashMachine - Uninitialise everything */
void TrashMachine(void)
{

 // textout(screen,font,MESSAGE[language][shut_down],0,0,1);

 destroy_bitmap(XBuf);

 if (OSD_MESSAGE_SPR)
   destroy_bitmap(OSD_MESSAGE_SPR);

 if (EAGLE_buf)
   destroy_bitmap(EAGLE_buf);

 if (sound_driver==1)
  set_volume(0,0);

  (*fade_out_proc[rand()%nb_fadeout])(0,0,vwidth,vheight);

 {
  UInt32 x;
  for (x=0;x<VRAMSIZE/32;x++)
          destroy_bitmap(dirty_spr[x]);
  }

 set_gfx_mode(GFX_TEXT,0,0,0,0);

 TrashSound();

 return ;
}


/* InitMachine - Starts up all the stuff */
int InitMachine(void)
{

 int result;

 Log("\n--[ INITIALISE MACHINE ]--------------------------\n");
 check_cpu();
 if (cpu_cpuid)
   Log("Following cpu info are exact\n");
 else
   Log("Following cpu info may be inaccurate\n");

 Log("Machine is a %d56, sub model : %d\n",cpu_family,cpu_model);

 if (cpu_fpu)
   Log("Machine got a fpu\n");
 else
   Log("No fpu available\n");

 if (cpu_mmx)
   Log("Machine got a MMX cpu\n");
 else
   Log("Machine doesn't have a MMX cpu\n");

 if (cpu_3dnow)
   Log("Machine got a 3dNow! cpu compatible\n");
 else
   Log("Machine doesn't have a 3dNow! cpu compatible\n");

 Log("Vendor : %s\n",cpu_vendor);


 set_gfx_mode(GFX_TEXT,0,0,0,0);
 printf(MESSAGE[language][init_allegro]);

 printf(MESSAGE[language][translated_by]);

 if (!(XBuf=create_bitmap(XBUF_WIDTH,XBUF_HEIGHT)))
  {
   printf(MESSAGE[language][failed_init]);
   return (0); 
  }

 printf(MESSAGE[language][clear_buffer]);
 clear(XBuf);

#ifndef LINUX  
  
 switch (gamepad_driver)
   {
    case 0:
         result = install_joystick(JOY_TYPE_NONE);
         break;
    case 1:
         result = install_joystick(JOY_TYPE_STANDARD);
         break;
    case 2:
         result = install_joystick(JOY_TYPE_2PADS);
         break;
    case 3:
         result = install_joystick(JOY_TYPE_4BUTTON);
         break;
    case 4:
         result = install_joystick(JOY_TYPE_6BUTTON);
         break;
    case 5:
         result = install_joystick(JOY_TYPE_8BUTTON);
         break;
    case 6:
         result = install_joystick(JOY_TYPE_FSPRO);
         break;
    case 7:
         result = install_joystick(JOY_TYPE_WINGEX);
         break;
    case 8:
         result = install_joystick(JOY_TYPE_SIDEWINDER);
         break;
    case 9:
         result = install_joystick(JOY_TYPE_GAMEPAD_PRO);
         break;
    case 10:
         result = install_joystick(JOY_TYPE_SNESPAD_LPT1);
         break;
    case 11:
         result = install_joystick(JOY_TYPE_SNESPAD_LPT2);
         break;
    case 12:
         result = install_joystick(JOY_TYPE_SNESPAD_LPT3);
         break;
    case 13:
         result = install_joystick(JOY_TYPE_WINGWARRIOR);
         break;

    default: // in particular, for gamepad_driver == -1
    result = install_joystick(JOY_TYPE_AUTODETECT);
   }

 if ((!result) && (joy_type!=JOY_TYPE_NONE))
        {
    Log("Joypad %d detected\n",joy_type);
         printf(MESSAGE[language][joy_detected]);
         gamepad=1;
         }
 else
        {
    Log("Joypad type %d NOT detected\n",joy_type);
         printf(MESSAGE[language][joy_no_detected]);
         gamepad=0;
         }
#endif

 Log("Allocating sprite array\n");
 printf(MESSAGE[language][alloc_spr]);
 {
   UInt32 x;
   for (x=0;x<VRAMSIZE/32;x++)
                {
                 if (!(dirty_spr[x]=create_bitmap(16,16)))
                        {
                          printf(MESSAGE[language][failed_init]);
                          return 0;
         }
        clear(dirty_spr[x]);
                 }
  }

 Log("Initiating sound\n");
 printf(MESSAGE[language][init_sound]);
 InitSound();
  
  if (use_eagle)
   {
    printf(MESSAGE[language][eagle_asked]);
    if (!set_gfx_mode(GFX_AUTODETECT,640,480,0,0))
            {
             vwidth=640;
             vheight=480;
             blit_x=(320-256)/2;
             blit_y=(240-216)/2;
             screen_blit_x=(WIDTH-io.screen_h)/2;
             screen_blit_y=(HEIGHT-io.screen_w)/2;
             SetPalette();

             EAGLE_buf=create_bitmap(640,480);

             }
    else
      printf(MESSAGE[language][eagle_mode_not_init]);
    }
  else
 if (!(*init_normal_mode[video_driver])())
        {
         Log("Can't set graphic mode\n");
         printf(MESSAGE[language][cant_set_gmode]);
         return 0;
         }
/*
            if ((vmode==0) && (!set_hugo_mode(256,240,256,240)));
            else
              if ((vmode<=1) && (!set_hugo_mode(256,256,256,240)));
            else
              if ((vmode<=2) && (!set_hugo_mode(320,200,256,240)));
            else
            {
             printf(MESSAGE[language][cant_set_gmode]);
             return 0;
             }*/
 // textout(screen,font,"OK\n Allocating colors...\n",0,0,1);


 text_mode(-1);
 install_keyboard();
 clear(screen);
 Log("End of initialisation of the machine\n");
 return 1;
}
/*
unsigned short savepict()
  {
          BITMAP *bmp;
          PALETTE pal;
          char* final="PICT0000.PCX";
          short unsigned tmp=0;

          while ((tmp!=0xFFFF) && (exists(final)))
              sprintf(final,"PICT%04X.PCX",++tmp);
          if (tmp==0xFFFF)
            return tmp;

          get_palette(pal);
          if (use_eagle)
          bmp = create_sub_bitmap(screen, (320-io.screen_w), 1 ,
                                                                                         io.screen_w*2,
                                                                                         io.screen_h*2);
          else
          bmp = create_sub_bitmap(screen, blit_x, blit_y,
                                                                                         io.screen_w,
                                                                                         io.screen_h);
          save_bitmap(final, bmp, pal);
//          save_bitmap(final, XBuf, pal);
          destroy_bitmap(bmp);
#ifndef FINAL_RELEASE
          fprintf(stderr,"picture saved %s\n",final);
#endif
          return tmp;

   }
// save a picture named PICTXXXX.PCX where XXXX is good so that it doesn't
// erase any file
*/

/*****************************************************************************

    Function:  set_message

    Description: create the sprite to display on screen for messages
    Parameters:char* mess, a pointer to the message to display
    Return:nothing

*****************************************************************************
void set_message(char* mess)
{

 if (MESSAGE_SPR)
   destroy_bitmap(MESSAGE_SPR);

 MESSAGE_SPR=create_bitmap(text_length(font,mess)+1,text_height(font)+1);
 clear(MESSAGE_SPR);
 textout(MESSAGE_SPR,font,mess,1,1,3);
 textout(MESSAGE_SPR,font,mess,0,0,255);
 return;
 }
*/

/*****************************************************************************

    Function:  set_hugo_mode

    Description: change the video mode
    Parameters: mode : mode of video screen
                width, height : minimum size of screen required
    Return: 0 on success
                 1 on error

*****************************************************************************
int set_hugo_mode(int mode,int width,int height)
{

 if (!set_gfx_mode(mode,width,height,0,0))
            {
             Log("Changing video mode to %dx%d\nPCE screen size is %dx%d\n",
                 width,
                 height,
                 io.screen_w,
                 io.screen_h);
             vwidth=width;
             vheight=height;

             blit_x=(width-io.screen_w)/2;
             blit_y=(height-io.screen_h)/2;

             screen_blit_x=(WIDTH-io.screen_w)/2;
             screen_blit_y=(224-io.screen_h)/2;

             Log("Now blit_x = %d\nblit_y = %d\nscreen_blit_x = %d\nscreen_blit_y = %d\n\n",
                 blit_x,
                 blit_y,
                 screen_blit_x,
                 screen_blit_y);

             SetPalette();
             return 0;
             }
 return 1;
 }
*/
 
 #ifndef LINUX
/*****************************************************************************

    Function:  init_normal_mode_normal

    Description: initialize the classic 256*224 video mode for normal video_driver
    Parameters: none
    Return: 0 on error
                 1 on success

*****************************************************************************
int init_normal_mode_normal()
{
  if (vmode==5)
    return !set_hugo_mode(GFX_AUTODETECT,800,600);

  if (io.screen_w==40*8)
    {
      if (!set_hugo_mode(GFX_AUTODETECT,320,240))
        return 1;
    }

  if (io.screen_w>40*8)
    {
      if (!set_hugo_mode(GFX_AUTODETECT,360,240))
        return 1;
    }

  if ((vmode==0) && (!set_hugo_mode(GFX_VGA,256,224)));
   else
  if ((vmode<=1) && (!set_hugo_mode(GFX_AUTODETECT,256,240)));
   else
  if ((vmode<=2) && (!set_hugo_mode(GFX_AUTODETECT,256,256)));
   else
  if ((vmode<=3) && (!set_hugo_mode(GFX_AUTODETECT,320,240)));
   else
  if ((vmode<=4) && (!set_hugo_mode(GFX_AUTODETECT,320,200)));
   else
          return 0;

 return 1;
 }

#else
/*****************************************************************************

    Function:  init_normal_mode_normal

    Description: initialize Linux unique screen mode (for now at least!)
    Parameters: none
    Return: 0 on error
                 1 on success

*****************************************************************************
int init_normal_mode_normal()
{
  if (set_hugo_mode(GFX_MODEX,320,240))
     return 0;
  
 return 1;
 }
*/
#endif
 
 
/*****************************************************************************

    Function: init_normal_mode_eagle

    Description:initialize the video screen but for use with eagle
    Parameters:none
    Return: 0 on error
            1 on success

*****************************************************************************/
int init_normal_mode_eagle()
{
  if (!(set_gfx_mode(GFX_AUTODETECT,640,480,0,0)))
            {
             vwidth=640;
             vheight=480;
             blit_x=(320-io.screen_w);
             blit_y=(240-io.screen_h) + 1;
             screen_blit_x=(WIDTH-io.screen_h)/2;
             screen_blit_y=(HEIGHT-io.screen_w)/2;

             SetPalette();
             return 1;
             }
  return 0;

}

/*****************************************************************************

    Function: init_normal_mode_scanline

    Description: init the screen mode to use when scanline mode enabled
    Parameters: none
    Return: 0 on error
            1 on success

*****************************************************************************/
int init_normal_mode_scanline()
{
  if (!(set_gfx_mode(GFX_AUTODETECT,640,480,0,0)))
            {
             vwidth=640;
             vheight=480;
             blit_x=(320-io.screen_w);
             blit_y=(240-io.screen_h);
             screen_blit_x=(WIDTH-io.screen_h)/2;
             screen_blit_y=(HEIGHT-io.screen_w)/2;

             SetPalette();
             return 1;
             }
  return 0;
 }



#else // ALLEGRO section ended
  #error include "svga.h" not this one!!!!!!
#endif
