#ifndef _INCLUDE_SYS_DEP_H
#define _INCLUDE_SYS_DEP_H

#include "cleantyp.h"

/* This header include all functionalities needed to be implemented in order
 * to make a port of Hu-Go! on various platforms
 */

/*
 * CD acces section
 * 
 * No need to implement them if you don't have a CD machine, don't want
 * or can't add PC Engine CD support
 */
  
  /*
   * osd_cd_init
   * 
   * Try to initialize the cd drive
   * The char* in input is the iso_filename, i.e. the name specified by the
   * -i command line parameter or "" if not specified (use default for your
   * system then)
   * return 0 on success else non zero value on error
   */
  int osd_cd_init(char*);

  /*
   * osd_cd_close
   * 
   * Releases the resources allocated by osd_cd_init
   */
  void osd_cd_close();


  /*
   * osd_cd_read
   * 
   * Read a 2048 cooked sector from the cd
   * Returns the read data in the UChar* variable
   * Data are located at sector in the UInt32 variable
   * return 0 on success else non zero value on error
   */
  int osd_cd_read(UChar*, UInt32);

  /*
   * osd_cd_stop_audio
   * 
   * Stops the audio playing from cd is any
   */
  void osd_cd_stop_audio();

  /*
   * osd_cd_track_info
   * 
   * Return the msf location of the beginning of the given track
   * plus the control byte of it
   */
  void osd_cd_track_info(UChar track,
                         int* min,
                         int* sec,
                         int* fra,
                         int* control);

  /*
   * osd_cd_nb_tracks
   * 
   * Return the index of the first and last track of the cd
   */
  void osd_cd_nb_tracks(int* first,
                        int* last);

  /*
   * osd_cd_length
   *
   * Return the total length of the cd
   */
  void osd_cd_length(int* min,
                     int* sec,
                     int* fra);


  /*
   * osd_cd_play_audio_track
   * 
   * Plays the selected audio track
   */
  void osd_cd_play_audio_track(UChar track);

  /*
   * osd_cd_play_audio_range
   * 
   * Plays audio range specified in msf mode
   */
  void osd_cd_play_audio_range(UChar min_from,
                               UChar sec_from,
                               UChar fra_from,
                               UChar min_to,
                               UChar sec_to,
                               UChar fra_to);

/*
 * Input section
 * 
 * This one function part implements the input functionality
 * It's called every vsync I think, i.e. almost 60 times a sec
 */

  /*
   * osd_keyboard
   *
   * Updates the Joypad variables, save/load game, make screenshots, display
   * gui or launch fileselector if asked
   * return 1 if we must quit the emulation
   * else 0
   */
  int osd_keyboard();

/*
 * Gfx section
 * 
 * Certainly one of the most important one, this one deals with all what can
 * be displayed.
 */

  /*
   * osd_gfx_buffer
   *
   * In order to make sprite and tiles functions generic to all ports, there's
   * a need for a linear pointer needed to access the graphic buffer.
   * This buffer will be the one used to display stuff on screen when calling
   * the osd_gfx_blit function.
   * I did this because e.g. when I use Allegro, I represent the buffer as a
   * XBuf BITMAP (it's an allegro type) and set the osd_gfx_buffer to
   * XBuf->lines[0] since it's the first byte of the REAL data in this bitmap.
   * Its size must be OSD_GFX_WIDTH * OSD_GFX_HEIGHT
   */
   extern UChar* osd_gfx_buffer;

  /*
   * osd_gfx_init_mode
   *
   * This function must set the video mode accordingly to the choosen user
   * desire and current PC Engine screen size.
   * In Dos port, it is either normal, eagle and scanline mode e.g.
   * return 0 in case of success and 1 else
   */
   int osd_gfx_init_mode();

  /*
   * osd_gfx_init_normal_mode
   *
   * This function inits anew (or for the first time) the video card so that
   * we can work in the waited screen mode even after a gui display or debug
   * screen or such..
   * return 0 in case of success and 1 else
   */
   int osd_gfx_init_normal_mode();

  /*
   * osd_gfx_set_color
   *
   * Set the 'index' color to components r,b,g
   */
   void osd_gfx_set_color(UChar index,
                       UChar r,
                       UChar g,
                       UChar b);


  /*
   * osd_gfx_savepict
   *
   * Saves the current screen bitmap, returns the numerical part of the
   * resulting filename
   */
   UInt16 osd_gfx_savepict();

  /*
   * osd_gfx_set_hugo_mode
   *
   * Asks to set up a screen sized width x height, using the mode 'mode'
   * which can be a driver in fact (X, SVGA, VESA, etc...)
   */
   SInt32 osd_gfx_set_hugo_mode(SInt32 mode,SInt32 width,SInt32 height);

/*
 * Miscellaneous section
 *
 * Here are function that don't belong to a set of function. It doesn't mean
 * they shouldn't be implemented but rather that they are important enough
 * to have their own section :)
 */

  /*
   * osd_init_machine
   *
   * This function is called once at the very beginning of the emulation and
   * must set once for all certain variables.
   * e.g. it should allocate mem for the osd_gfx_buffer variable, init joypad
   * stuff e.g. or anything else that will last the whole emulation session.
   * return 0 on success else non zero value
   */
   int osd_init_machine();

#endif
